from leapp.actors import Actor
from leapp.dialogs import Dialog
from leapp.models import InstalledOracleSignedRPM
from leapp.libraries import stdlib
from leapp.libraries.stdlib import api
from leapp.dialogs.components import BooleanComponent
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag
from leapp import reporting
from leapp.reporting import Report, create_report
import os
import sys


class CheckRDMAUEKR5(Actor):
    """
    Check if running UEKR5 and RDMA and confirm upgrade to UEKR6 in OL8

    """

    name = 'check_rdma_uekr5'
    consumes = (InstalledOracleSignedRPM,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)
    dialogs = (
        Dialog(
            scope='RDMA_upgrade_kernel_to_UEKR6',
            reason='Confirmation',
            components=(
                BooleanComponent(
                    key='confirm',
                    label='Confirm kernel upgrade to UEKR6 for RDMA? '
                          'If no, the upgrade process will be interrupted.',
                    description='Confirm upgrade kernel to UEKR6 for RDMA.',
                    reason='Only UEKR6 kernel is available on OL8. '
                ),
            )
        ),
    )

    def process(self):
        kernel_uekr5 = self.is_kernel_uekr5()
        libibverbs_upgrade = self.check_libibverbs_upgrade()
        if kernel_uekr5 is True and libibverbs_upgrade is True:
                result = self.confirm()
                if result is True:
                    self.produce_report()
                else:
                    self.produce_inhibitor()

    def is_kernel_uekr5(self):
        """
        Is running kernel UEKR5
        """
        try:
            running_kernel = stdlib.run(['uname', '-r'])['stdout'].strip()
        except (OSError, stdlib.CalledProcessError):
            api.current_logger().warning('Failed to query uname for running kernel', exc_info=True)
            return

        api.current_logger().info('Active kernel found {}'.format(running_kernel))
        if (running_kernel.find('4.14.35') != -1):
            return True
        else:
            return False

    def check_libibverbs_upgrade(self):
        pkgs = self.get_pkgs("libibverbs")
        if not pkgs:
            api.current_logger().info('Package libibverbs not found')
            return False
        for pkg in pkgs:
            release = pkg.release
            api.current_logger().info('libibverbs release found {}'.format(release))
            if (release.find('el7ora') != -1):
                return True
            else:
                return False
            if (release.find('el7vos') != -1):
                return True
            else:
                return False

    def get_pkgs(self,pkg_name):
        """
        Get all installed packages of the given name signed by Oracle.
        """
        rpms = next(api.consume(InstalledOracleSignedRPM), InstalledOracleSignedRPM()).items
        return [pkg for pkg in rpms if pkg.name == pkg_name]

    def confirm(self):
        answer = self.get_answers(self.dialogs[0])
        return answer.get('confirm')

    def produce_report(self):
        create_report([
            reporting.Title('Confirm kernel upgrade to UEKR6 for RDMA?'),
            reporting.Summary(
                'Detected active UEKR5 kernel and RDMA, confirmed that kernel will be upgraded to UEKR6 in OL8'
            ),
            reporting.ExternalLink(
                title='Unbreakable Enterprise Kernel Release 6',
                url='https://docs.oracle.com/en/operating-systems/uek/6/relnotes6.2/ol_instav.html#uek6_install_rdma'
            ),
            reporting.Severity(reporting.Severity.HIGH)
        ])

    def produce_inhibitor(self):
        create_report([
            reporting.Title('Confirm kernel upgrade to UEKR6 for RDMA?'),
            reporting.Summary(
                'Upgrade process was interrupted because UEKR5 is the active kernel '
                'and RDMA packages are installed. Need to confirm upgrade of kernel '
                'to UEKR6 in OL8.'
            ),
            reporting.ExternalLink(
                title='Unbreakable Enterprise Kernel Release 6',
                url='https://docs.oracle.com/en/operating-systems/uek/6/relnotes6.2/ol_instav.html#uek6_install_rdma'
            ),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Groups([
                    reporting.Groups.SECURITY,
                    reporting.Groups.TOOLS
            ]),
            reporting.Groups([reporting.Groups.INHIBITOR]),
            reporting.Remediation(hint='Confirm kernel upgrade to UEKR6 for RDMA?')
        ])

